package ru.dbotthepony.mc.otm.block.entity;

import com.google.common.collect.ImmutableList;
import net.minecraft.core.BlockPos;
import net.minecraft.core.Direction;
import net.minecraft.nbt.CompoundTag;
import net.minecraft.network.chat.Component;
import net.minecraft.network.chat.TranslatableComponent;
import net.minecraft.world.entity.player.Inventory;
import net.minecraft.world.entity.player.Player;
import net.minecraft.world.inventory.AbstractContainerMenu;
import net.minecraft.world.level.Level;
import net.minecraft.world.level.block.state.BlockState;
import net.minecraftforge.common.capabilities.Capability;
import net.minecraftforge.common.util.LazyOptional;
import net.minecraftforge.items.CapabilityItemHandler;
import ru.dbotthepony.mc.otm.matter.IMatterGridCell;
import ru.dbotthepony.mc.otm.matter.MatterGrid;
import ru.dbotthepony.mc.otm.Registry;
import ru.dbotthepony.mc.otm.capability.IMatterHandler;
import ru.dbotthepony.mc.otm.capability.IPatternStorage;
import ru.dbotthepony.mc.otm.capability.MatteryCapability;
import ru.dbotthepony.mc.otm.container.MatteryContainer;
import ru.dbotthepony.mc.otm.menu.PatternStorageMenu;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import java.util.ArrayList;
import java.util.Collection;
import java.util.concurrent.atomic.AtomicInteger;

public class BlockEntityPatternStorage extends BlockEntityMattery implements IMatterGridCell, IPatternStorage {
	public BlockEntityPatternStorage(BlockPos p_155229_, BlockState p_155230_) {
		super(Registry.BlockEntities.PATTERN_STORAGE, p_155229_, p_155230_);
	}

	public final MatteryContainer patterns = new MatteryContainer(this::setChangedPatterns, 3 * 3) {
		@Override
		public int getMaxStackSize() {
			return 1;
		}
	};

	private void setChangedPatterns() {
		setChanged();

		if (grid != null)
			grid.notifyPatternsChanged();
	}

	private MatterGrid grid;

	@Override
	public CompoundTag save(CompoundTag nbt) {
		nbt.put("patterns", patterns.serializeNBT());
		return super.save(nbt);
	}

	@Override
	public void load(CompoundTag nbt) {
		super.load(nbt);
		patterns.deserializeNBT(nbt.get("patterns"));
	}

	@Override
	public void setLevel(Level p_155231_) {
		super.setLevel(p_155231_);

		if (grid == null)
			MatterGrid.scheduleDiscoverNeighbours(this, getBlockPos(), p_155231_);
	}

	@Nullable
	@Override
	public MatterGrid getMatterGrid() {
		return grid;
	}

	@Nullable
	@Override
	public IMatterHandler getMatterHandler() {
		return null;
	}

	@Nullable
	@Override
	public IPatternStorage getPatternStorage() {
		return valid ? this : null;
	}

	private boolean valid = true;

	@Override
	public boolean isValidMatterCell() {
		return valid;
	}

	@Override
	public void setMatterGrid(MatterGrid grid) {
		this.grid = grid;
	}

	@Override
	public void invalidateCaps() {
		super.invalidateCaps();
		valid = false;
	}

	@Override
	public void reviveCaps() {
		super.reviveCaps();
		valid = true;
	}

	private LazyOptional<BlockEntityPatternStorage> resolver = LazyOptional.of(() -> this);

	@Nonnull
	@Override
	public <T> LazyOptional<T> getCapability(@Nonnull Capability<T> cap, @Nullable Direction side) {
		if (cap == MatteryCapability.PATTERN || cap == CapabilityItemHandler.ITEM_HANDLER_CAPABILITY)
			return resolver.cast();

		return super.getCapability(cap, side);
	}

	private static final TranslatableComponent NAME = new TranslatableComponent("block.overdrive_that_matters.pattern_storage");

	@Override
	protected Component getDefaultDisplayName() {
		return NAME;
	}

	@Nullable
	@Override
	public AbstractContainerMenu createMenu(int containerID, Inventory inventory, Player ply) {
		return new PatternStorageMenu(containerID, inventory, this);
	}

	@Override
	public Collection<PatternState> getStoredPatterns() {
		ArrayList<PatternState> list = new ArrayList<>();
		patterns.consumeCapability(MatteryCapability.PATTERN, capability -> list.addAll(capability.getStoredPatterns()));
		return ImmutableList.copyOf(list);
	}

	@Override
	public int getCapacity() {
		AtomicInteger stored = new AtomicInteger();
		patterns.consumeCapability(MatteryCapability.PATTERN, capability -> stored.getAndAdd(capability.getCapacity()));
		return stored.get();
	}

	@Override
	public int getStored() {
		AtomicInteger stored = new AtomicInteger();
		patterns.consumeCapability(MatteryCapability.PATTERN, capability -> stored.getAndAdd(capability.getStored()));
		return stored.get();
	}

	@Override
	public boolean insertPattern(PatternState pattern, boolean only_update, boolean simulate) {
		for (IPatternStorage storage : patterns.capabilityIterator(MatteryCapability.PATTERN)) {
			if (storage.insertPattern(pattern, only_update, simulate)) {
				if (!simulate) {
					setChangedPatterns();
				}

				return true;
			}
		}

		return false;
	}
}
