package ru.dbotthepony.mc.otm.capability;

import net.minecraft.world.item.Item;

import javax.annotation.Nonnull;
import java.util.Collection;
import java.util.function.Predicate;
import java.util.stream.Collectors;

public interface IPatternStorage {
	record PatternState(Item item, double research_percent) {
		public PatternState(@Nonnull Item item, double research_percent) {
			this.item = item;
			this.research_percent = Math.max(0, Math.min(1, research_percent));
		}

		public boolean equals(PatternState state) {
			return state.item == this.item && state.research_percent == this.research_percent;
		}
	}

	/**
	 * @return unmodifiable collection of stored patterns
	 */
	Collection<PatternState> getStoredPatterns();

	default Collection<PatternState> findPattern(Item item) {
		return findPattern((item2) -> item.equals(item2.item));
	}

	default Collection<PatternState> findPattern(Predicate<PatternState> predicate) {
		return getStoredPatterns().stream().filter(predicate).collect(Collectors.toList());
	}

	default boolean hasExactPattern(PatternState state) {
		for (PatternState state1 : getStoredPatterns()) {
			if (state1.equals(state)) {
				return true;
			}
		}

		return false;
	}

	int getCapacity();
	int getStored();

	/**
	 * @param pattern pattern to be inserted or update value from
	 * @param only_update do not insert new pattern if no existing patterns match the pattern
	 * @param simulate whenever to affect state
	 * @return whenever operation was successful
	 */
	boolean insertPattern(PatternState pattern, boolean only_update, boolean simulate);

	default boolean insertPattern(PatternState pattern, boolean simulate) {
		return insertPattern(pattern, false, simulate);
	}

	default boolean updatePattern(PatternState pattern, boolean simulate) {
		return insertPattern(pattern, true, simulate);
	}
}
