package ru.dbotthepony.mc.otm.matter;

import net.minecraft.core.BlockPos;
import net.minecraft.core.Direction;
import net.minecraft.core.SectionPos;
import net.minecraft.world.level.Level;
import net.minecraft.world.level.block.entity.BlockEntity;
import net.minecraft.world.level.chunk.LevelChunk;
import ru.dbotthepony.mc.otm.capability.IMatterHandler;
import ru.dbotthepony.mc.otm.capability.IPatternStorage;

import javax.annotation.Nullable;

public interface IMatterGridCell {
	@Nullable
	MatterGrid getMatterGrid();

	@Nullable
	IMatterHandler getMatterHandler();

	@Nullable
	IPatternStorage getPatternStorage();

	boolean isValidMatterCell();

	void setMatterGrid(MatterGrid grid);

	default void notifyPatternsChanged() {

	}

	default void scheduleDiscoverNeighbours(BlockPos pos, Level level) {
		MatterGrid.scheduleDiscoverNeighbours(this, pos, level);
	}

	default boolean connectOrCreateMatterGrid(BlockPos pos, Level level) {
		return connectOrCreateMatterGrid(pos, level, false);
	}

	default void onNeighbourMatterCell(BlockPos pos, Level level, Direction direction, IMatterGridCell cell) {

	}

	default boolean connectOrCreateMatterGrid(BlockPos pos, Level level, boolean force) {
		if (getMatterGrid() != null && !force)
			return true;

		boolean full_discovery = true;

		for (Direction direction : Direction.values()) {
			BlockPos offset = pos.offset(direction.getNormal());

			// level.getBlockEntity can suck big cuks of deadlocks
			LevelChunk get_chunk = level.getChunkSource().getChunkNow(SectionPos.blockToSectionCoord(offset.getX()), SectionPos.blockToSectionCoord(offset.getZ()));

			if (get_chunk == null) {
				full_discovery = false;
				continue;
			}

			BlockEntity get_entity = get_chunk.getBlockEntity(offset);

			if (get_entity != this && get_entity instanceof IMatterGridCell cell) {
				MatterGrid grid = cell.getMatterGrid();

				if (grid != null && grid != getMatterGrid()) {
					if (getMatterGrid() == null) {
						grid.track(this);
					} else {
						grid.mergeWith(getMatterGrid());
					}
				}

				if (cell.isValidMatterCell())
					onNeighbourMatterCell(pos, level, direction, cell);
			}
		}

		if (getMatterGrid() == null) {
			new MatterGrid().track(this);
		}

		return full_discovery;
	}
}
